@@ -3,22 +3,22 @@
 #' Fills the background color of a column using a gradient based on
 #' the values given by an expression
 #'
-#' The syntax in condformat rules has changed since v0.7. See \code{\link{rule_fill_gradient_old}}
+#' The syntax in condformat rules has changed since v0.7. See [rule_fill_gradient_old]
 #'
 #' @family rule
 #'
-#' @param x A condformat object, typically created with `condformat(x)`
+#' @param x A condformat object, typically created with [condformat()]
 #' @param columns A character vector with column names to be coloured. Optionally
-#'                `tidyselect::select_helpers` can be used.
+#'                [tidyselect::select_helpers()] can be used.
 #' @param expression an expression to be evaluated with the data.
-#'                   It should evaluate to a logical or an integer vector,
-#'                   that will be used to determine which cells are to be coloured.
+#'                   It should evaluate to a numeric vector,
+#'                   that will be used to determine the colour gradient level.
 #' @inheritParams scales::seq_gradient_pal
 #' @param limits range of limits that the gradient should cover
 #' @param na.value fill color for missing values
 #' @param lockcells logical value determining if no further rules should be applied to the affected cells.
 #'
-#' @param ... Dots are used to transition from the old syntax \code{\link{rule_fill_discrete_old}} to the new one
+#' @param ... Dots are used to transition from the old syntax [rule_fill_discrete_old] to the new one
 #'
 #' @return The condformat_tbl object, with the added formatting information
 #' @examples
@@ -32,25 +32,13 @@
 #'   rule_fill_gradient(starts_with("Sepal"), expression=Sepal.Length - Sepal.Width)
 #'
 #' @export
-rule_fill_gradient <- function(...) {
-  quoted_args <- rlang::quos(...)
-  condformat_api <- "0.6"
-  tryCatch({
-    possible_condformat <- quoted_args[[1]]
-    x <- rlang::eval_tidy(possible_condformat)
-    stopifnot(inherits(x, "condformat_tbl"))
-    condformat_api <- "0.7"
-  }, error = function(err) {
-    condformat_api <- "0.6"
-  })
-  if (condformat_api == "0.7") {
-    return(rule_fill_gradient_new(...))
-  } else if (condformat_api == "0.6") {
-    warning("This condformat syntax is deprecated. See ?rule_fill_gradient for more information")
-    return(rule_fill_gradient_old(...))
-  } else {
-    stop("Unknown condformat API")
-  }
+rule_fill_gradient <- function(x, columns, expression,
+                               low = "#132B43", high = "#56B1F7",
+                               space = "Lab",
+                               na.value = "#7F7F7F",
+                               limits = NA,
+                               lockcells = FALSE, ...) {
+  return(api_dispatcher(rule_fill_gradient_new, rule_fill_gradient_old))
 }
 
 
@@ -105,13 +93,15 @@ rule_fill_gradient_old <- function(...,
   return(rule)
 }
 
-#' @rdname rule_fill_gradient
 rule_fill_gradient_new <- function(x, columns, expression,
                                    low = "#132B43", high = "#56B1F7",
                                    space = "Lab",
                                    na.value = "#7F7F7F",
                                    limits = NA,
                                    lockcells = FALSE) {
+  if (!inherits(x, "condformat_tbl")) {
+    x <- condformat(x)
+  }
   columnsquo <- rlang::enquo(columns)
   helpers <- tidyselect::vars_select_helpers
   columnsquo_bur <- rlang::env_bury(columnsquo, !!! helpers)
@@ -175,60 +165,64 @@ rule_fill_gradient_ <- function(columns,
   return(rule)
 }
 
-applyrule.rule_fill_gradient <- function(rule, finalformat, xfiltered, xview, ...) {
-  if (inherits(rule$expression, "lazy")) {
+rule_to_cf_field.rule_fill_gradient <- function(rule, xfiltered, xview, ...) {
+  if (inherits(rule[["expression"]], "lazy")) {
     # Deprecated
-    columns <- dplyr::select_vars_(colnames(xview), rule$columns) # D
-    values_determining_color <- lazyeval::lazy_eval(rule$expression, xfiltered) # D
+    columns <- dplyr::select_vars_(colnames(xview), rule[["columns"]]) # D
+    values_determining_color <- lazyeval::lazy_eval(rule[["expression"]], xfiltered) # D
     values_determining_color <- rep(values_determining_color, length.out = nrow(xfiltered))
-    rule_fill_gradient_common(rule, finalformat, xview, columns, values_determining_color)
+    r_f_g_to_cf_field_common(rule, xview, columns, values_determining_color)
   } else {
-    columns <- tidyselect::vars_select(colnames(xview), !!! rule$columns)
+    columns <- tidyselect::vars_select(colnames(xview), !!! rule[["columns"]])
     if (length(columns) == 0) {
-      return(finalformat)
+      return(NULL)
     }
-    if (rlang::quo_is_missing(rule$expression)) {
+    if (rlang::quo_is_missing(rule[["expression"]])) {
       if (length(columns) > 1) {
         warning("rule_fill_gradient applied to multiple columns, using column ",
                 columns[1], " values as expression. In the future this behaviour will change,",
                 " please use a explicit expression instead.",
                 call. = FALSE)
       }
-      rule$expression <- as.symbol(as.name(columns[1]))
+      rule[["expression"]] <- as.symbol(as.name(columns[1]))
     }
-    values_determining_color <- rlang::eval_tidy(rule$expression, data = xfiltered)
+    values_determining_color <- rlang::eval_tidy(rule[["expression"]], data = xfiltered)
     values_determining_color <- rep(values_determining_color, length.out = nrow(xfiltered))
-    rule_fill_gradient_common(rule, finalformat, xview, columns, values_determining_color)
+    r_f_g_to_cf_field_common(rule, xview, columns, values_determining_color)
   }
 }
 
-applyrule.rule_fill_gradient_ <- function(rule, finalformat, xfiltered, xview, ...) {
+rule_to_cf_field.rule_fill_gradient_ <- function(rule, xfiltered, xview, ...) {
   # Deprecated
-  columns <- dplyr::select_vars_(colnames(xview), rule$columns) # D
-  values_determining_color <- lazyeval::f_eval(f = rule$expression, data = xfiltered) # D
+  columns <- dplyr::select_vars_(colnames(xview), rule[["columns"]]) # D
+  values_determining_color <- lazyeval::f_eval(f = rule[["expression"]], data = xfiltered) # D
   values_determining_color <- rep(values_determining_color, length.out = nrow(xfiltered))
-  rule_fill_gradient_common(rule, finalformat, xview, columns, values_determining_color)
+  r_f_g_to_cf_field_common(rule, xview, columns, values_determining_color)
 }
 
 #' @importFrom scales rescale
-rule_fill_gradient_common <- function(rule, finalformat, xview,
-                                      columns, values_determining_color) {
-  if (identical(rule$limits, NA)) {
+r_f_g_to_cf_field_common <- function(rule, xview,
+                                     columns, values_determining_color) {
+  if (identical(rule[["limits"]], NA)) {
     limits <- range(values_determining_color, na.rm = TRUE)
   } else {
-    limits <- rule$limits
+    limits <- rule[["limits"]]
   }
 
-  col_scale <- scales::seq_gradient_pal(low = rule$low, high = rule$high, space = rule$space)
+  col_scale <- scales::seq_gradient_pal(low = rule[["low"]], high = rule[["high"]], space = rule[["space"]])
 
   values_rescaled <- scales::rescale(x = values_determining_color, from = limits)
   colours_for_values <- col_scale(values_rescaled)
   stopifnot(identical(length(colours_for_values), nrow(xview)))
-  colours_for_values <- matrix(colours_for_values,
-                               nrow = nrow(xview), ncol = ncol(xview), byrow = FALSE)
-
-  finalformat <- fill_css_field_by_cols(finalformat, "background-color",
-                                        colours_for_values, columns,
-                                        xview, rule$lockcells)
-  return(finalformat)
+  colours_for_values_mat <- matrix(NA,
+                                   nrow = nrow(xview), ncol = ncol(xview),
+                                   byrow = FALSE)
+  colnames(colours_for_values_mat) <- colnames(xview)
+  colours_for_values_mat[, columns] <- colours_for_values
+  cf_field <- structure(list(css_key = "background-color",
+                             css_values = colours_for_values_mat,
+                             lock_cells = rule[["lockcells"]]),
+                        class = c("cf_field_rule_fill_solid",
+                                  "cf_field_css", "cf_field"))
+  return(cf_field)
 }
