@@ -7,10 +7,7 @@ condformat(iris[c(1:5,70:75, 120:125),]) %>%
                      expression = Sepal.Width > Sepal.Length - 2.25,
                      colours = c("TRUE" = "#7D00FF")) %>%
   rule_fill_gradient2(Petal.Length) %>%
-  rule_css(Sepal.Length,
-           expression = ifelse(Species == "setosa", "bold", "regular"),
-           css_field = "font-weight") %>%
-  rule_css(Sepal.Length,
-           expression = ifelse(Species == "setosa", "yellow", "black"),
-           css_field = "color")
+  rule_text_bold(Sepal.Length, Species == "setosa") %>%
+  rule_text_color(Sepal.Length, ifelse(Species == "setosa", "yellow", "")) %>%
+  rule_fill_bar(Petal.Width, limits = c(0, NA))
 
